# VisdomUtils.cmake - plugin utility functions for the cmake build system
# Taken mostly from Paraview. This is not yet that featureful, but for the start this should work

SET (CMAKE_LIBRARY_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/bin CACHE INTERNAL
  "Single output directory for building all libraries.")
SET (CMAKE_RUNTIME_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/bin CACHE INTERNAL
  "Single output directory for building all executables.")
SET(PLUGIN_SCHEMA_DIR ${CMAKE_RUNTIME_OUTPUT_DIRECTORY}/schema)
SET(CORE_SCHEMA_DIR ${PLUGIN_SCHEMA_DIR}/core)

define_property(SOURCE PROPERTY XSD_ARGS
	BRIEF_DOCS "custom xsd compiler arguments"
	FULL_DOCS "custom xsd compiler arguments"
)

function(visdom_build_optional_plugin name comment subdirectory default)
	OPTION(VISDOM_BUILD_PLUGIN_${name} "Build ${comment}" ${default})
	IF(VISDOM_BUILD_PLUGIN_${name})
		# I'm not sure whether we want to place the files like this:  "${CMAKE_CURRENT_BINARY_DIR}/${name}")
		ADD_SUBDIRECTORY("${subdirectory}") 
		MESSAGE(STATUS "Plugin: ${comment} enabled")
	ELSE(VISDOM_BUILD_PLUGIN_${name})
		MESSAGE(STATUS "Plugin: ${comment} disabled")
	ENDIF(VISDOM_BUILD_PLUGIN_${name})
endfunction(visdom_build_optional_plugin)

function(visdom_add_plugin_test plugin_name)
	include_directories(${TESTS_DIRECTORY})
	set(_test_name "test_${plugin_name}")
	add_executable(${_test_name} ${ARGN})
	#set_target_properties(${name} PROPERTIES RUNTIME_OUTPUT_DIRECTORY ${TESTS_DIRECTORY})
	target_link_libraries(${_test_name} ${plugin_name} server datastructures)
	add_test(${_test_name} ${CMAKE_RUNTIME_OUTPUT_DIRECTORY}/${_test_name})
endfunction(visdom_add_plugin_test)

macro(visdom_declare_plugin name category_name)
	SET(PLUGIN_PATH ${CMAKE_LIBRARY_OUTPUT_DIRECTORY})
	configure_file(${CMAKE_SOURCE_DIR}/CMake/pluginPath.h ${CMAKE_CURRENT_BINARY_DIR}/pluginPath.h)
	set(visdom_current_plugin ${name})
	set(visdom_current_plugin_category ${category_name})
	set(visdom_current_plugin_sources)
endmacro(visdom_declare_plugin)

function(visdom_scan_sources sources generated_sources generated_include)
	foreach(current_src ${ARGN})
		string(REGEX REPLACE ".*\\." "" extension "${current_src}" )
		string(TOLOWER ${extension} lower_case_extension)
		
		if(lower_case_extension STREQUAL "xsd")
			get_source_file_property(args "${current_src}" XSD_ARGS)
			if(NOT args)
				set(args ${XSD_ARGS})
			endif(NOT args)
			WRAP_XSD(XSDS_SRCS XSDS_INCLUDE ${CMAKE_RUNTIME_OUTPUT_DIRECTORY}/schema/plugins/${visdom_current_plugin} ${current_src} OPTIONS ${args})
			set_source_files_properties(${current_src} PROPERTIES HEADER_FILE_ONLY true)
			set(generated_sources_p ${generated_sources_p} ${XSDS_SRCS})
			set(${generated_include} ${XSDS_INCLUDE} PARENT_SCOPE)
		endif(lower_case_extension STREQUAL "xsd")
		
		set(sources_p ${sources_p} ${current_src})
	endforeach(current_src)
	set(${sources} ${sources_p} PARENT_SCOPE)
	set(${generated_sources} ${generated_sources_p} PARENT_SCOPE)
endfunction(visdom_scan_sources)

function(visdom_add_assets)
	foreach(asset ${ARGN})
	    CONFIGURE_FILE(${asset} ${CMAKE_RUNTIME_OUTPUT_DIRECTORY}/assets/${visdom_current_plugin}/${asset} COPY_ONLY)
	endforeach(asset)
endfunction(visdom_add_assets)

function(visdom_add_additional_sources name)
	visdom_scan_sources(sources generated_sources includes ${ARGN})
	set(visdom_current_plugin_sources ${visdom_current_plugin_sources} ${sources} PARENT_SCOPE)
	set(visdom_current_plugin_generated_sources ${visdom_current_plugin_generated_sources} ${generated_sources} PARENT_SCOPE)
	source_group(${name} FILES ${sources})
endfunction(visdom_add_additional_sources)

function(visdom_add_node name)
	SET(_MODE "0")
	SET(first_xsd "1")
	SET(other_xsd_args ${XSD_ARGS})
	SET(GENERATE_SOURCE_GROUP "1")
	
	foreach(current_arg ${ARGN})
		IF(${current_arg} STREQUAL "SOURCES")
			SET(_MODE "1")
		elseif(${current_arg} STREQUAL "ASSETS")
			SET(_MODE "2")
		elseif(${current_arg} STREQUAL "NODEINCLUDE")
			SET(_MODE "3")
		elseif(${current_arg} STREQUAL "NO_SOURCE_GROUP")
			SET(GENERATE_SOURCE_GROUP "0")
		elseif(${current_arg} STREQUAL "TESTS")
			SET(_MODE "4")
		else(${current_arg} STREQUAL "SOURCES")
			if(_MODE STREQUAL "1")
				set(sources ${sources} ${current_arg})
			elseif(_MODE STREQUAL "2")
				set(assets ${assets} ${current_arg})
			elseif(_MODE STREQUAL "3")
				set(node_include ${current_arg})
			elseif(_MODE STREQUAL "4")
				set(node_tests ${node_tests} ${current_arg})
			endif(_MODE STREQUAL "1")
		endif(${current_arg} STREQUAL "SOURCES")
	endforeach(current_arg)
	
	visdom_scan_sources(output_sources output_generated_sources generated_includes ${sources})

	if(${GENERATE_SOURCE_GROUP})
		source_group(${name} FILES ${output_sources})
	endif(${GENERATE_SOURCE_GROUP})

	set(visdom_current_plugin_sources ${visdom_current_plugin_sources} ${output_sources} PARENT_SCOPE)
	set(visdom_current_plugin_generated_sources ${visdom_current_plugin_generated_sources} ${output_generated_sources} PARENT_SCOPE)
	set(visdom_current_plugin_includes ${visdom_current_plugin_includes} ${generated_includes} PARENT_SCOPE)
	set(visdom_current_plugin_tests ${visdom_current_plugin_tests} ${node_tests} PARENT_SCOPE)

	set(visdom_current_plugin_node_includes "${visdom_current_plugin_node_includes}#include <${node_include}>\n" PARENT_SCOPE)
	set(visdom_current_plugin_main_src "${visdom_current_plugin_main_src}\tfactory->register_node<${name}>(string(\"${visdom_current_plugin_category}.${name}\"), getBinPath() + \"/schema/plugins/${visdom_current_plugin}/\" + ${name}::getSettingsXsdName());\n" PARENT_SCOPE)
	set(visdom_current_plugin_exit_src "${visdom_current_plugin_exit_src}\tfactory->unregister_node(string(\"${visdom_current_plugin_category}.${name}\"), getBinPath() + \"/schema/plugins/${visdom_current_plugin}/\" + ${name}::getSettingsXsdName());\n" PARENT_SCOPE)
	
endfunction(visdom_add_node)

function(visdom_finalize_plugin)
	configure_file(${CMAKE_SOURCE_DIR}/CMake/plugin_main.cpp.in ${CMAKE_CURRENT_BINARY_DIR}/plugin_main.cpp)
	SOURCE_GROUP("Object Files" REGULAR_EXPRESSION ".obj$" )
	SOURCE_GROUP("Generated Sources" FILES ${visdom_current_plugin_generated_sources} ${CMAKE_CURRENT_BINARY_DIR}/plugin_main.cpp)
	
	if(visdom_current_plugin_tests)
		configure_file(${CMAKE_SOURCE_DIR}/CMake/test_plugin_main.cpp.in ${CMAKE_CURRENT_BINARY_DIR}/test_plugin_main.cpp)
		visdom_add_plugin_test(${visdom_current_plugin} ${CMAKE_CURRENT_BINARY_DIR}/test_plugin_main.cpp ${visdom_current_plugin_tests})
	endif(visdom_current_plugin_tests)

	INCLUDE_DIRECTORIES(
		${CMAKE_CURRENT_BINARY_DIR} 
		${CMAKE_CURRENT_SOURCE_DIR}
		${visdom_current_plugin_includes} 
		${STD_INCLUDE_DIRS}
	)

	SET(BUILD_SHARED_LIBS ON)
	CUDA_ADD_LIBRARY(${visdom_current_plugin} 
		${visdom_current_plugin_sources} 
		${visdom_current_plugin_generated_sources}
		${CMAKE_CURRENT_BINARY_DIR}/plugin_main.cpp
	)
	target_link_libraries(${visdom_current_plugin} utils dataflow server datastructures ${COMMON_LIBS})
	# http://gcc.gnu.org/faq.html#dso
	# We open our plugins using dlopen. Therefore (assuming not win32 equals to having gcc), 
	# we need to pass -E to the linker, and add RTLD_GLOBAL to dlopen. 
	# Make sure boost-extension is patched for this
	if(NOT WIN32)
		SET_TARGET_PROPERTIES(${visdom_current_plugin} PROPERTIES LINK_FLAGS -Wl,-E)
	endif(NOT WIN32)
endfunction(visdom_finalize_plugin)

SET(VISDOM_EXTRA_EXTERNAL_PLUGINS "" CACHE FILEPATH "Select Extra Plugins to build with Visdom")

FOREACH(external ${VISDOM_EXTRA_EXTERNAL_PLUGINS})
	FIND_PATH(${external}_SOURCE_DIR "dummystring.txt")
	IF(EXISTS ${${external}_SOURCE_DIR}/CMakeLists.txt)
		visdom_build_optional_plugin(${external} "User Plugin" ${${external}_SOURCE_DIR} "ON")
	ENDIF(EXISTS ${${external}_SOURCE_DIR}/CMakeLists.txt)
ENDFOREACH(external)


